;
; Line timings difference between VGA and NTSC using IRQ_LINE to change backdrop color. This test is for real hardware only
; and doesn't produce any difference between the video modes or correct timings on the current emulators!
;
; Programmed by: doslogo
; Copyright (C) 2025 doslogo
;
; Disclaimer: The code below is written to get a test going as quick as possible to run on the real hardware, and the code is not 
;             stable to use in any game without heavy register management and synchronization methods! This code is not meant as 
;             a teaching case!
;

.define USE_GREEN_COLOR		0	; Uses green backdrop instead of red if that is preferred

SCNALINE_TEST			= (218*2); Scanline number to test (we want an even number because 240p, but this is a value between 0-511)
	.if ADD_NOPS=1
X_COORDINATE_LINE		= 311	; Where we except the color change on the line (depends on how long it takes to get to the vblank code)
	.else
X_COORDINATE_LINE		= 120	; Where we except the color change on the line (depends on how long it takes to get to the vblank code)
	.endif
Y_COORDINATE_LINE		= 218	; Where we think the scanline actually is at (we can dream)


.segment "CODE"
	cld							; Clear decimal mode

	stz $9F25					; DCSEL=0 for this entire test!

	;
	; Let's set VGA or NTSC-240p based on the preprocessor definition VGA:
	;
.if VGA=0
	lda #%01001010				; 240p, NTSC output, and sprites
.else
	lda #%01001001				; 240p, VGA, and sprites
.endif

	sta $9F29

	; 240p means 320x240 resolution!
	lda #$40
	sta $9F2A
	sta $9F2B

	;
	; Get an ISR up to do vblank and hblank for us
	;
	php
	sei							; Disable interrupts

	lda #<isr
	sta $0314+0
	lda #>isr
	sta $0314+1

	; Turn on vblank, vblank will turn on hblank
	lda #%00000001
	sta $9F26

	plp							; Restore interrupts

	;
	; Need a sprite on the scanline we are interested in, because it is difficult to count scanlines on a regular CRT with composite.
	; Turn on the mouse, and force its coordinates to the scanline we care about
	;
	lda #1
	ldx #0
	ldy #0
	jsr $FF68

	stz $9F25
	lda #$02					; Sprite 0 X coordinate address $1FC02
	sta $9F20					; 
	lda #$FC
	sta $9F21
	lda #(%00010000|$01)		; Increment 1 byte and high bit of VRAM address
	sta $9F22

	lda #<X_COORDINATE_LINE		; X coordinate of mouse cursor tip
	sta $9F23
	lda #>X_COORDINATE_LINE		; X coordinate of mouse cursor tip
	sta $9F23
	lda #Y_COORDINATE_LINE		; Y coordinate of mouse cursor tip
	sta $9F23

	;
	; No more to do here, everything is done with the ISR!
	;
@:	bra @-


;
; Interrupt Service Routine:
;
isr:
	; Act fast, only recognize vblank, else it is hblank (this might not be true)!
	lda $9F27
	and #%00000001
	beq @hblank

	jmp @vblank

@hblank:

	.if ADD_NOPS=1
		nop							; 63*2 cycles of wait
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	.endif

	; Set the backdrop color to black. This is the timing visual trigger!
	stz $9F23
	stz $9F23
	
	; Disable hblank, only do vblank
	lda #%00000001
	sta $9F26

@exit_isr:
	; Ack IRQs!
	lda #%00000011				; Signal VERA we are done with its IRQs
	sta $9F27

	;
	; Return from interrupt (skipping kernal, so no keyboard, no turning off led etc!)
	;
	ply
	plx
	pla
	rti

@vblank:
	;
	; Hblank must have priority and speed for this test, so set VERA addresses now
	;
	stz $9F25
	stz $9F20					; backdrop color address $1FA00
	lda #$FA
	sta $9F21
	lda #(%00010000|$01)		; Increment 1 byte and high bit of VRAM address
	sta $9F22

	; Set the backdrop color to something visible, like red
.if USE_GREEN_COLOR=1
	lda #$F0
	sta $9F23
	stz $9F23
.else
	stz $9F23
	lda #$0F
	sta $9F23
.endif

	; Reset backdrop so we don't have to in vblank
	stz $9F20					; backdrop color address $1FA00

	; Now, wait for horizontal blank to set the backdrop color to black. In an ideal world, this would happen 
	; on the previous scanline of the interrupted line and outside the visible screen
	;
	; Let's choose scanline SCNALINE_TEST, which can be weird if the scanline is set to even of odd and scroll registers are written to
	lda #<SCNALINE_TEST
	sta $9F28
	lda #<(%00000011|((SCNALINE_TEST&$100)>>1))	; Top bit of scanline and enable hblank+vblank
	sta $9F26

	jmp @exit_isr
